@@ -15,7 +15,7 @@
 #include "webserver.h"
 #include "WebPages.h"
 
-#define VERSION 2.3
+#define VERSION 2.4
 
 extern char *getfqdn(char *);
 extern void trim(char *);
@@ -126,16 +126,18 @@ int HandleWebPages(char *HTTPBuffer, char *HTTPOutputBuffer, int size,
 
 int Status_Page(char *source, char *dest, int size)
 {
-	strncpy(dest,
-		"<HTML><BODY><H1><CENTER>Welcome to SNARE for Solaris version 2.3</H1><P>Please select from the menu on the left.</CENTER><p>",
-		size);
+	if(source && dest && size) {
+		strncpy(dest,
+			"<HTML><BODY><H1><CENTER>Welcome to SNARE for Solaris version 2.4</H1><P>Please select from the menu on the left.</CENTER><p>",
+			size);
 
-	// Ok.. we're cheating here. The only way we can get here is if we're running.
-	strncat(dest,
-		"Status: <font color=green>Snare for Solaris currently running.</font>",
-		size - strlen(dest));
+		// Ok.. we're cheating here. The only way we can get here is if we're running.
+		strncat(dest,
+			"Status: <font color=green>Snare for Solaris currently running.</font>",
+			size - strlen(dest));
 
-	strncat(dest, "</BODY></HTML>", size - strlen(dest));
+		strncat(dest, "</BODY></HTML>", size - strlen(dest));
+	}
 	return (0);
 }
 
@@ -143,7 +145,7 @@ int ShowUsers(int http_socket)
 {
 	struct passwd *user;
 	struct spwd *sp;
-	int retval;
+	int retval=0;
 	char temp[1024];
 	char hostid[MAX_HOSTID];
 
@@ -215,7 +217,7 @@ int ShowUsers(int http_socket)
 int ShowGroups(int http_socket)
 {
 	struct group *groupp;
-	int retval;
+	int retval=0;
 	char temp[1024];
 	char hostid[MAX_HOSTID];
 
@@ -238,36 +240,105 @@ int ShowGroups(int http_socket)
 int ShowGroupMembers(int http_socket)
 {
 	struct group *groupp;
-	int retval;
+	struct passwd *userp;
+	int retval=0;
 	char temp[1024];
 	int firstmember = 1;
 	char hostid[MAX_HOSTID];
 
+	struct UNode {
+		char username[256];
+		int gid;
+		struct UNode *next;
+	};
+
+	struct UNode *head=NULL;
+	struct UNode *current=NULL;
+	struct UNode *tail=NULL;
+	int found=0;
+
 	getfqdn(hostid);
 
 	snprintf(temp, sizeof (temp), "HOST: %s\n", hostid);
-	retval = Send(http_socket, temp, strlen(temp));
+	retval += Send(http_socket, temp, strlen(temp));
+
+	// HERE: Pull data out of the password file, looking
+	// for users with a primary group of this group.
+	// add them in.
+	// What about duplicates?
+	// Linked list would be a bit of an an overkill.. but I need to use one.
+	setpwent();
+	// Look through the password file for users with this pgid
+	while((userp = (struct passwd *) getpwent()) != (struct passwd *) NULL) {
+		// Zap up a simple linked list
+		tail=malloc(sizeof(struct UNode));
+		strncpy(tail->username,userp->pw_name,sizeof(tail->username));
+		tail->gid=userp->pw_gid;
+		tail->next = NULL;
+		if(!head) {
+			head=tail;
+		}
+		if(current) {
+			current->next=tail;
+		}
+		current=tail;
+	}
+	endpwent();
+
 	setgrent();
 	while ((groupp = (struct group *) getgrent()) != (struct group *) NULL) {
 		firstmember = 1;
 		snprintf(temp, sizeof (temp), "%s\t%d\t", groupp->gr_name,
-			 (int)groupp->gr_gid);
-		retval = Send(http_socket, temp, strlen(temp));
-		while (*(groupp->gr_mem) != NULL) {
-			if (firstmember) {
-				snprintf(temp, sizeof (temp), "%s",
-					 *((groupp->gr_mem)++));
-				firstmember = 0;
-			} else {
-				snprintf(temp, sizeof (temp), ",%s",
-					 *((groupp->gr_mem)++));
+			(int)groupp->gr_gid);
+		retval += Send(http_socket, temp, strlen(temp));
+
+		// Push out our linke list users first
+		current=head;
+		while(current) {
+			if(current->gid == groupp->gr_gid) {
+				if(firstmember) {
+					firstmember=0;
+				} else {
+					retval+=Send(http_socket,",",1);
+				}
+				retval+=Send(http_socket,current->username,strlen(current->username));
 			}
-			retval = Send(http_socket, temp, strlen(temp));
+			current=current->next;
 		}
-		retval = Send(http_socket, "\n", strlen("\n"));
+
+		while (*(groupp->gr_mem) != NULL) {
+			// Scan through our linked list to avoid duplicates.
+			found=0;
+			current=head;
+			while(current) {
+				if(current->gid == groupp->gr_gid && strcmp(*(groupp->gr_mem), current->username) == 0) {
+					// Already printed.
+					found=1;
+				}
+				current=current->next;
+			}
+			if(!found) {
+				// New user
+				if(firstmember) {
+					firstmember=0;
+				} else {
+					retval+=Send(http_socket,",",1);
+				}
+				retval+=Send(http_socket,*(groupp->gr_mem),strlen(*(groupp->gr_mem)));
+			}
+			groupp->gr_mem++;
+		}
+		retval += Send(http_socket, "\n", strlen("\n"));
 	}
 	endgrent();
 
+	// Clear our temporary linked list.
+	while(head) {
+		current=head->next;
+		free(head);
+		head=current;
+	}
+
 	return (1);
 }
 
@@ -278,6 +349,10 @@ int Network_Config(char *source, char *dest, int size)
 	char str_DestPort[10];
 	char str_conferr[10], str_neterr[10];
 
+	if(!source || !dest || !size) {
+		return(0);
+	}
+
 	strcpy(config_struct.str_ClientName, "");
 	strcpy(config_struct.str_FileDestination, "");
 	strcpy(config_struct.str_NetworkDestination, "");
@@ -586,6 +661,10 @@ int Remote_Config(char *source, char *dest, int size)
 	char str_WebPort[10];
 	char str_remerr[10];
 
+	if(!source || !dest || !size) {
+		return(0);
+	}
+
 	dw_remote_error = Read_Remote_From_File(&remote_struct);
 
 	// This function will display the form used to set the remote audit configuration
@@ -675,7 +754,9 @@ int Remote_Set(char *source, char *dest, int size)
 	struct Reg_Remote remote_struct;
 	int dw_error = 0;
 
-	//This function will display that the remote audit configurations have been changed, or there have been errors
+	dw_error = Read_Remote_From_File(&remote_struct);
+
+	// This function will display that the remote audit configurations have been changed, or there have been errors
 	strncpy(dest, "<h1><center>SNARE Remote Control Configuration</h1>",
 		size);
 
@@ -705,8 +786,10 @@ int Remote_Set(char *source, char *dest, int size)
 				sizeof (remote_struct.str_RestrictIP));
 		}
 		if (strstr(Variable, "str_Password") != NULL) {
-			strncpy(remote_struct.str_Password, Argument,
-				sizeof (remote_struct.str_Password));
+			if(strcmp(Argument,remote_struct.str_Password)!=0) {
+				strncpy(remote_struct.str_Password,Argument,sizeof(remote_struct.str_Password));
+
+			}
 		}
 
 		if (strstr(Variable, "str_OldPassword") != NULL) {
@@ -816,9 +899,18 @@ int Remote_Set(char *source, char *dest, int size)
 						    dw_Password) {
 							if(strcmp(remote_struct.str_Password,OldPassword)) {
 								// The password has changed. Encrypt it.
-								fprintf(configfile,
-									"	accesskey=%s\n",
-									(char *)crypt(remote_struct.str_Password,SALT));
+								char *tpass;
+								tpass=crypt(remote_struct.str_Password,SALT);
+								if(tpass) {
+									fprintf(configfile,
+										"	accesskey=%s\n",
+										tpass);
+								} else {
+									// Crypt failed. Fallback to whatever the user had before.
+									fprintf(configfile,
+										"	accesskey=%s\n",
+										remote_struct.str_Password);
+								}
 							} else {
 								// Otherwise, leave it in peace
 								fprintf(configfile,
@@ -906,6 +998,10 @@ int Objective_Config(char *source, char *dest, int size)
 
 	FILE *configfile = (FILE *) NULL;
 
+	if(!source || !dest || !size) {
+		return(0);
+	}
+
 	strncpy(dest,
 		"<form action=setobjective><H1><CENTER>SNARE Filtering Objectives Configuration</H1>",
 		size);
@@ -919,7 +1015,7 @@ int Objective_Config(char *source, char *dest, int size)
 
 		strncat(dest,
 			"<tr bgcolor=#FFFFBB><center><td><b>Action Required</b></td><td><b>Criticality</b></td>"
-			"<td><b>Event ID Match</b></td><td><b>User Match</b></td><td><b>Search Term (regular expression)</b>"
+			"<td><b>Event ID Match</b></td><td><b>User Match</b></td><td><b>Search Term</b>"
 			"</td></center></tr>", size - strlen(dest));
 
 		while (Get_Next_Objective(configfile, &reg_objective)) {
@@ -1083,13 +1179,16 @@ int Objective_Display(char *source, char *dest, int size)
 				sizeof (reg_objective.str_event_type));
 			strncpy(reg_objective.str_critic, CRITICAL_TOKEN,
 				sizeof (reg_objective.str_critic));
-			strncpy(reg_objective.str_eventid_match, LOGONOFF_TOKEN,
+			strncpy(reg_objective.str_eventid_match, LOGON_LOGOFF_EVENTS,
 				sizeof (reg_objective.str_eventid_match));
 			strncpy(reg_objective.str_general_match,
-				"\"Enter the non-header search term here\"",
+				"Enter the non-header search term here",
 				sizeof (reg_objective.str_general_match));
+			strncpy(reg_objective.str_user_match_type,
+				"Any",
+				sizeof (reg_objective.str_user_match_type));
 			strncpy(reg_objective.str_user_match,
-				"\"Enter the user match here\"",
+				".*",
 				sizeof (reg_objective.str_user_match));
 		}
 
@@ -1159,6 +1258,15 @@ int Objective_Display(char *source, char *dest, int size)
 		strncat(dest,
 			">Modify system, file or directory attributes  </td><td><input type=radio name=str_eventid_match value=",
 			size - strlen(dest));
+		strncat(dest, FILE_REMOVE_TOKEN, size - strlen(dest));
+		if (strcmp(reg_objective.str_eventid_match, FILE_REMOVE_EVENTS)
+		    == 0) {
+			strncat(dest, " checked", size - strlen(dest));
+			i_set = 1;
+		}
+		strncat(dest,
+			">Remove a file or directory  </td><td><input type=radio name=str_eventid_match value=",
+			size - strlen(dest));
 		strncat(dest, PROCESS_TOKEN, size - strlen(dest));
 		if (strcmp(reg_objective.str_eventid_match, PROCESS_EVENTS)
 		    == 0) {
@@ -1166,7 +1274,7 @@ int Objective_Display(char *source, char *dest, int size)
 			i_set = 1;
 		}
 		strncat(dest,
-			">Start or stop program execution  </td><td><input type=radio name=str_eventid_match value=",
+			">Start or stop program execution  </td></tr><tr><td><input type=radio name=str_eventid_match value=",
 			size - strlen(dest));
 		strncat(dest, USER_TOKEN, size - strlen(dest));
 		if (strcmp(reg_objective.str_eventid_match, USER_EVENTS) ==
@@ -1175,7 +1283,7 @@ int Objective_Display(char *source, char *dest, int size)
 			i_set = 1;
 		}
 		strncat(dest,
-			">Change user or group identity   </td></tr><tr><td><input type=radio name=str_eventid_match value=",
+			">Change user or group identity   </td><td><input type=radio name=str_eventid_match value=",
 			size - strlen(dest));
 		strncat(dest, SOCKET_TOKEN, size - strlen(dest));
 		if (strcmp(reg_objective.str_eventid_match, SOCKET_EVENTS)
@@ -1191,7 +1299,7 @@ int Objective_Display(char *source, char *dest, int size)
 			i_set = 1;
 		}
 		strncat(dest,
-			">Any event(s) </td><td></td></tr></table></td></tr>",
+			">Any event(s) </td></tr></table></td></tr>",
 			size - strlen(dest));
 
 		strncat(dest,
@@ -1202,7 +1310,7 @@ int Objective_Display(char *source, char *dest, int size)
 		strncat(dest, "\"></td></tr>", size - strlen(dest));
 
 		strncat(dest,
-			"<tr bgcolor=#FFFFBB><td>Search Term (regular expression) </td><td><input type=text name=str_general_match size=50 value=\"",
+			"<tr bgcolor=#FFFFBB><td>Search Term<br><i>(regular expression)</i></td><td><input type=text name=str_general_match size=50 value=\"",
 			size - strlen(dest));
 		strncat(dest, reg_objective.str_general_match,
 			size - strlen(dest));
@@ -1212,23 +1320,33 @@ int Objective_Display(char *source, char *dest, int size)
 			"<tr bgcolor=#FFFFCC><td>Select the User Match Type</td><td>",
 			size - strlen(dest));
 		strncat(dest,
-			"<input type=radio name=str_user_match_type value=",
+			"<input type=radio name=str_user_match_type value=Any",
 			size - strlen(dest));
-		strncat(dest, "Include", size - strlen(dest));
-		if (strstr(reg_objective.str_user_match_type, "Exclude") ==
-		    NULL)
+
+		if (strcmp(reg_objective.str_user_match, ".*") ==
+		    NULL) {
 			strncat(dest, " checked", size - strlen(dest));
+			strncpy(reg_objective.str_user_match_type,"Any",sizeof(reg_objective.str_user_match_type));
+		}
+		strncat(dest, ">Match Any User    ", size - strlen(dest));
 		strncat(dest,
-			">Include    <input type=radio name=str_user_match_type value=",
+			"<input type=radio name=str_user_match_type value=Include",
 			size - strlen(dest));
-		strncat(dest, "Exclude", size - strlen(dest));
+		if (strstr(reg_objective.str_user_match_type, "Include") !=
+		    NULL) {
+			strncat(dest, " checked", size - strlen(dest));
+		}
+		strncat(dest,
+			">Include    <input type=radio name=str_user_match_type value=Exclude",
+			size - strlen(dest));
+
 		if (strstr(reg_objective.str_user_match_type, "Exclude") !=
 		    NULL)
 			strncat(dest, " checked", size - strlen(dest));
 		strncat(dest, ">Exclude    </td></tr>", size - strlen(dest));
 
 		strncat(dest,
-			"<tr bgcolor=#FFFFCC><td>User Search Term </td><td><input type=text name=str_user_match size=50 value=\"",
+			"<tr bgcolor=#FFFFCC><td>User Search Term<br><i>(comma separated user list)</i> </td><td><input type=text name=str_user_match size=50 value=\"",
 			size - strlen(dest));
 		strncat(dest, reg_objective.str_user_match,
 			size - strlen(dest));
@@ -1266,10 +1384,6 @@ int Objective_Display(char *source, char *dest, int size)
 		// Identify the log type to capture. Note that there is a table within a table in these radio buttons.
 		i_set = 0;
 
-		strncat(dest,
-			"<tr bgcolor=#FFFFCC><td>Identify the event logs <br>(ignored if any objective other <br>than 'Any event(s)' is selected):</td><td><table  width=100% border=0><tr>",
-			size - strlen(dest));
-
 		strncat(dest,
 			"<tr bgcolor=#FFFFBB><td>Select the Alert Level</td><td>",
 			size - strlen(dest));
@@ -1572,10 +1686,13 @@ int Objective_Result(char *source, char *dest, int size)
 	}
 
 	if (!i_event_type_set) {
+		strncat(reg_objective.str_event_type,
+			SUCCESS_TOKEN,
+			sizeof (reg_objective.str_event_type) -
+			strlen(reg_objective.str_event_type));
 		strncat(dest,
-			"<br>Either Success or Failure (or both) mosut be specified. Please hit your 'Back' button to return to the Objective Configuration page.<p>",
+			"<br>Either Success or Failure (or both) must be specified. I have defaulted this objective to SUCCESS.<p>",
 			size - strlen(dest));
-		return (1);
 	}
 
 	if (strstr(str_eventid_radio, "Any_Event") != NULL) {
@@ -1601,6 +1718,11 @@ int Objective_Result(char *source, char *dest, int size)
 			strncpy(reg_objective.str_eventid_match,
 				FILE_ATTRIB_EVENTS,
 				sizeof(reg_objective.str_eventid_match));
+		} else if (strstr(str_eventid_radio, FILE_REMOVE_TOKEN) !=
+			   NULL) {
+			strncpy(reg_objective.str_eventid_match,
+				FILE_REMOVE_EVENTS,
+				sizeof(reg_objective.str_eventid_match));
 		} else if (strstr(str_eventid_radio, PROCESS_TOKEN) !=
 			   NULL) {
 			strncpy(reg_objective.str_eventid_match,
@@ -1668,6 +1790,7 @@ int Objective_Result(char *source, char *dest, int size)
 						if (headertype ==
 						    CONFIG_OBJECTIVES) {
 							int critic = 0;
+							char userstring[8];
 							// WRITE OUT NEW OBJECTIVE HERE
 							if (strstr
 							    (reg_objective.
@@ -1704,17 +1827,39 @@ int Objective_Result(char *source, char *dest, int size)
 								    CRITICALITY_CLEAR;
 							}
 
+
+							if (strstr(reg_objective.str_user_match_type,"Any") != NULL) {
+								strncpy(userstring, "user", sizeof(userstring));
+								strncpy(reg_objective.str_user_match,".*",sizeof(reg_objective.str_user_match));
+							} else {
+								char TUserMatch[SIZE_OF_USERMATCH+5];
+								char *pTUM=TUserMatch;
+
+								snprintf(TUserMatch,sizeof(TUserMatch),"^(%s)$",reg_objective.str_user_match);
+								while(*pTUM) {
+									if(*pTUM==',') {
+										*pTUM='|';
+									}
+
+									pTUM++;
+								}
+								strncpy(reg_objective.str_user_match,TUserMatch,sizeof(reg_objective.str_user_match));
+								
+								if (strstr(reg_objective.str_user_match_type,"Include") != NULL) {
+									strncpy(userstring, "user", sizeof(userstring));
+								} else {
+									strncpy(userstring, "user!", sizeof(userstring));
+								}
+							}
+
 							fprintf(configfile,
-								"	criticality=%d	event=%s	return=%s	user=%s	match=%s\n",
+								"	criticality=%d	event=%s	return=%s	%s=%s	match=%s\n",
 								critic,
-								reg_objective.
-								str_eventid_match,
-								reg_objective.
-								str_event_type,
-								reg_objective.
-								str_user_match,
-								reg_objective.
-								str_general_match);
+								reg_objective.str_eventid_match,
+								reg_objective.str_event_type,
+								userstring,
+								reg_objective.str_user_match,
+								reg_objective.str_general_match);
 
 							wroteconfig = 1;
 						}
@@ -1763,14 +1908,28 @@ int Objective_Result(char *source, char *dest, int size)
 						critic = CRITICALITY_CLEAR;
 					}
 
-					if (strstr
-					    (reg_objective.str_user_match_type,
-					     "Exclude") == NULL) {
-						strncpy(userstring, "user",
-							sizeof (userstring));
+					if (strstr(reg_objective.str_user_match_type,"Any") != NULL) {
+						strncpy(userstring, "user", sizeof(userstring));
+						strncpy(reg_objective.str_user_match,".*",sizeof(reg_objective.str_user_match));
 					} else {
-						strncpy(userstring, "user!",
-							sizeof (userstring));
+						char TUserMatch[SIZE_OF_USERMATCH+5];
+						char *pTUM=TUserMatch;
+
+						snprintf(TUserMatch,sizeof(TUserMatch),"^(%s)$",reg_objective.str_user_match);
+						while(*pTUM) {
+							if(*pTUM==',') {
+								*pTUM='|';
+							}
+
+							pTUM++;
+						}
+						strncpy(reg_objective.str_user_match,TUserMatch,sizeof(reg_objective.str_user_match));
+								
+						if (strstr(reg_objective.str_user_match_type,"Include") != NULL) {
+							strncpy(userstring, "user", sizeof(userstring));
+						} else {
+							strncpy(userstring, "user!", sizeof(userstring));
+						}
 					}
 
 					fprintf(configfile,
@@ -1837,6 +1996,7 @@ int Objective_Result(char *source, char *dest, int size)
 						    i_objective) {
 							// Replace this objective with the new version.
 							int critic = 0;
+							char userstring[8];
 							// WRITE OUT NEW OBJECTIVE HERE
 							if (strstr
 							    (reg_objective.
@@ -1873,17 +2033,39 @@ int Objective_Result(char *source, char *dest, int size)
 								    CRITICALITY_CLEAR;
 							}
 
+							if (strstr(reg_objective.str_user_match_type,"Any") != NULL) {
+								strncpy(userstring, "user", sizeof(userstring));
+								strncpy(reg_objective.str_user_match,".*",sizeof(reg_objective.str_user_match));
+							} else {
+								char TUserMatch[SIZE_OF_USERMATCH+5];
+								char *pTUM=TUserMatch;
+
+								snprintf(TUserMatch,sizeof(TUserMatch),"^(%s)$",reg_objective.str_user_match);
+								while(*pTUM) {
+									if(*pTUM==',') {
+										*pTUM='|';
+									}
+
+									pTUM++;
+								}
+								strncpy(reg_objective.str_user_match,TUserMatch,sizeof(reg_objective.str_user_match));
+								
+								if (strstr(reg_objective.str_user_match_type,"Include") != NULL) {
+									strncpy(userstring, "user", sizeof(userstring));
+								} else {
+									strncpy(userstring, "user!", sizeof(userstring));
+								}
+							}
+
 							fprintf(configfile,
-								"	criticality=%d	event=%s	return=%s	user=%s	match=%s\n",
+								"	criticality=%d	event=%s	return=%s	%s=%s	match=%s\n",
 								critic,
 								reg_objective.
 								str_eventid_match,
-								reg_objective.
-								str_event_type,
-								reg_objective.
-								str_user_match,
-								reg_objective.
-								str_general_match);
+								reg_objective.str_event_type,
+								userstring,
+								reg_objective.str_user_match,
+								reg_objective.str_general_match);
 
 							position += size;
 							objectivecounter++;
@@ -1930,11 +2112,11 @@ int Objective_Result(char *source, char *dest, int size)
 
 		if (dw_objective_error == 0)
 			strncat(dest,
-				"<br>The objective has been modifed/added.",
+				"<br>The objective has been modified/added.",
 				size - strlen(dest));
 		else
 			strncat(dest,
-				"<br>The objective was unable to be modifed/added.",
+				"<br>The objective was unable to be modified/added.",
 				size - strlen(dest));
 	}
 
@@ -1943,9 +2125,32 @@ int Objective_Result(char *source, char *dest, int size)
 
 int DefaultHeader(char *source, char *dest, int size)
 {
+
+	if(!source || !dest || !size) {
+		return(0);
+	}
+
 	strncpy(dest, "<HTML><head>"
 		"<title>InterSect Alliance - Information Technology Security</title>"
 		"<meta name=\"TITLE\" content=\"InterSect Alliance - Information Technology Security\">"
+		"<style type=\"text/css\">\n"
+		"body {\n"
+		" font-family: Verdana,Helvetica,sans-serif;\n"
+		" font-size: 10px; font-weight: normal;\n"
+		" margin: 0px;\n"
+		"}\n"
+		"font {\n"
+		" font-family: Verdana,Helvetica,sans-serif;\n"
+		" text-decoration: none; font-size: 10px;\n"
+		" font-weight: normal;\n"
+		"}\n"
+		"table {\n"
+		" margin: 0px; padding: 0px;\n"
+		"}\n"
+		"td {\n"
+		" font-size: 75%;\n"
+		"}\n"
+		"</style>\n"
 		"</head>"
 		"<body text=black bgcolor=white link=#000066 vlink=#000044 alink=#000055>"
 		"<table border=0 cellspacing=0 cellpadding=0 columns=3 width=100%>"
@@ -1967,7 +2172,7 @@ int DefaultHeader(char *source, char *dest, int size)
 		"<td height=70 width=1% bgcolor=#cc3333><br></td>"
 		"<td height=70 width=1% bgcolor=#cc2222><br></td>"
 		"<td height=70 width=1% bgcolor=#cc1111><br></td>"
-		"<td height=70 width=86% bgcolor=#cc0000><div align=right><font face=\"Helvetica,Arial,sans-serif\" size=+2 color=white><b><center>SNARE for Solaris&nbsp;</b></font><font color=white></font>&nbsp;&nbsp;</center></div></td>"
+		"<td height=70 width=86% bgcolor=#cc0000><font color=white><center><h1>SNARE for Solaris</h1></center></font></td>"
 		"</tr>"
 		"</tbody>"
 		"</table>"
@@ -2005,12 +2210,17 @@ int DefaultHeader(char *source, char *dest, int size)
 
 int DefaultFooter(char *source, char *dest, int size)
 {
+
+	if(!source || !dest || !size) {
+		return(0);
+	}
+
 	strncpy(dest, "</td>"
 		"</tr>"
 		"</tbody>"
 		"</table>"
 		"</body><center>"
-		"<BR><BR><FONT SIZE=-1 face=helvetica>(c) <A HREF=\"http://www.intersectalliance.com\">Intersect Alliance</A> Pty Ltd 2002. "
+		"<BR><BR><FONT SIZE=-1>(c) <A HREF=\"http://www.intersectalliance.com\">Intersect Alliance</A> Pty Ltd 1999-2004. "
 		"This site is powered by <A HREF=\"http://www.intersectalliance.com/projects/index.html\">SNARE for Solaris.</A></FONT>"
 		"</center></html>", size);
 
@@ -2032,6 +2242,10 @@ int Restart(char *source, char *dest, int size, int socketone, int sockettwo)
 {
 	int pid;
 
+	if(!source || !dest || !size) {
+		return(0);
+	}
+
 	switch (pid = fork()) {
 	case 0:
 		// Child process.
@@ -2062,6 +2276,10 @@ int InterSectImage(char *source, char *dest, int size)
 	char temp[2029];
 	int size2;
 
+	if(!source || !dest || !size) {
+		return(0);
+	}
+
 	strncpy(IntersectGif,
 		"R0lGODlhzQBGALMAAB8aF01JR2toZtwrGYF+fONVRo+Mi5eWleuCePKzrLOysszLy/jRzunl"
 		"5f////j29SwAAAAAzQBGAEAI/gAdCBxIsKDBgw4eIFxIUCHDhxAjSkT4AMGAAQgmatw40CHH"
@@ -2095,6 +2313,10 @@ int InterSectImage(char *source, char *dest, int size)
 
 	size2=base64decode (temp, IntersectGif);
 
+	if(size2>size) {
+		return(0);
+	}
+
 	memcpy(dest,temp,size2);
 	return(size2);
 }
@@ -2427,10 +2649,16 @@ int Get_Next_Objective(FILE * configfile, struct Reg_Objective *objective)
 	//int notreturnflag = 0;	// HERE: Still need to add in support for this.
 
 	while (fgets(inputbuffer, MAX_AUDIT_CONFIG_LINE, configfile)) {
+		trim(inputbuffer);
+
 		if (isheader(inputbuffer)) {
 			return (0);
 		}
 
+		if(strlen(inputbuffer) < 3) {
+			continue;
+		}
+
 		if ((criticality =
 		     splitobjective(inputbuffer, event, user, path,
 				    &returncode, &excludeflag)) > -1) {
@@ -2453,6 +2681,23 @@ int Get_Next_Objective(FILE * configfile, struct Reg_Objective *objective)
 					sizeof (objective->
 						str_user_match_type));
 			}
+
+			// Strip out our ^( and )$ characters
+			// .. and zap in commas
+			if(user[0]=='^' && user[1]=='(' && strlen(user) >= 4) {
+				int count;
+				int userlen=strlen(user);
+				if(user[userlen-2]==')' && user[userlen-1]=='$') {
+					user[userlen-2]='\0';
+					for(count=0;count < userlen-3;count++) {
+						user[count]=user[count+2];
+						if(user[count]=='|') {
+							user[count]=',';
+						}
+					}
+				}
+			}
+
 			strncpy(objective->str_user_match, user,
 				SIZE_OF_USERMATCH);
 			if (returncode == 1) {
