@@ -1,10 +1,10 @@
 /**********************************************************
  *
- * Snare for Solaris version 2.3
+ * Snare for Solaris version 2.4
  *
  * Author: InterSect Alliance Pty Ltd
  *
- * Copyright 2001-2003 InterSect Alliance Pty Ltd
+ * Copyright 2001-2004 InterSect Alliance Pty Ltd
  *
  * Last Modified: 5/7/2004
  *
@@ -58,8 +58,13 @@
  *                  information is read from the praudit pipe.
  *      22/12/2003  Starting integration of regcomp() into the linked list
  *                  data to save CPU.
- *      20/ 7/2004  Integrated MD5 password checksum capability
+ *      20/ 6/2004  Integrated MD5 password checksum capability
  *                  Changed the base64 decoding function to kill off a bug.
+ *      12/ 7/2004  Fixed service termination problem, added support
+ *                  for unencrypted passwords.
+ *      26/ 7/2004  Moving back to 'sledge hammer' approach to audit class
+ *                  configuration, as auditconfig -conf no longer sets
+ *                  process to class mappings properly.
  *
  **********************************************************
  *
@@ -68,6 +73,9 @@
  *
  **********************************************************/
 
+// Sledgehammer class mapping?
+#define SLEDGEHAMMER 1
+
 
 #include <sys/types.h> 
 #include <sys/stat.h> 
@@ -92,12 +100,20 @@
 #include <limits.h>
 #include <dirent.h>
 #include <netdb.h>
+#include <values.h>
+
+#ifdef SLEDGEHAMMER
+	// #include <sys/proc.h>
+
+	#include <procfs.h>
+#endif
 
 #include <regex.h>
 
 #include "snarecore.h"
 #include "webserver.h"
 
+
 ////// DEBUG
 #include <stdarg.h>
 void DebugMsg(const char* pszFormat, ...);
@@ -442,6 +458,7 @@ int main(int argc, char *argv[])
 						if(webpid) {
 							kill(webpid,SIGTERM);
 						}
+
 						kill(ppid,SIGTERM); waitpid(pid,NULL,NULL);
 						exit(1);
 					} else if(error == EINTR) {
@@ -467,18 +484,6 @@ int main(int argc, char *argv[])
 				}
 			} 
 		} else {
-			/* Setup a kill signal handler so that when the primary
-			   terminates, it closes nicely and takes the other forks
-			    down with it */
-			if(signal(SIGTERM,kill_signal) == SIG_ERR)
-			{
-				fprintf(stderr,"Cannot set signal SIGTERM\n");
-				audit_off();
-				kill(ppid,SIGTERM); waitpid(pid,NULL,NULL);
-				exit(1);
-			}
-
-			logbuffercount=0;
 			char *newline=logbuffer;
 			char *endbuffer = logbuffer+sizeof(logbuffer)-1;
 			char *bufferptr = logbuffer;
@@ -493,9 +498,22 @@ int main(int argc, char *argv[])
 			int returncode;
 
 			char *tbuffer;
+			/* Setup a kill signal handler so that when the primary
+			   terminates, it closes nicely and takes the other forks
+			    down with it */
+			if(signal(SIGTERM,kill_signal) == SIG_ERR)
+			{
+				fprintf(stderr,"Cannot set signal SIGTERM\n");
+				audit_off();
+				kill(ppid,SIGTERM); waitpid(pid,NULL,NULL);
+				exit(1);
+			}
+
+			logbuffercount=0;
+
 
 			// while ((bytesread=read(praudit2daemon[0],bufferptr,spaceavail) >0) && continueloop)
-			while (continueloop) {
+			while (continueloop && !caught_kill) {
 				if(spaceavail==0) {
 					// Oh dear.. filled our buffer without seeing a newline. Must be filled with corrupt data from praudit.
 					// Clear it.
@@ -561,7 +579,6 @@ int main(int argc, char *argv[])
 				/* Kill the other child process (auditsvc),
 				   and restart praudit and auditsvc */
 				audit_off();
-
 				kill(pid,SIGTERM);
 				/* Wait for the process to die */
 				waitpid(pid,NULL,NULL);
@@ -698,6 +715,12 @@ int setclass(unsigned int class)
 	int pid;
 	int pspid;
 
+#ifdef SLEDGEHAMMER
+	int psdata;
+	struct psinfo psbuf;
+	char fname[1024];
+#endif
+
 	DIR *procdir;
 	struct dirent *dp;            /* for readdir() return values */
 
@@ -723,34 +746,31 @@ int setclass(unsigned int class)
 
 		pinfo.ap_pid = pspid;
 
-/* THIS IS ANOTHER WAY TO DO A SIMILAR THING to the code below.
-   - HOWEVER, we need to be sure not to apply audit to
-     NSCD, as it seems to kill solaris.
-// DEBUG ONLY 
-#include <sys/proc.h>
-#include <procfs.h>
-{
-	int psdata;
-	struct psinfo psbuf;
-	char fname[1024];
+#ifdef SLEDGEHAMMER
+		/* THIS IS ANOTHER WAY TO DO A SIMILAR THING to the code below.
+   		   - HOWEVER, we need to be sure not to apply audit to
+     		     NSCD, as it seems to kill solaris. */
 
-	snprintf(fname,sizeof(fname),"/proc/%s/psinfo",dp->d_name);
- 	if( (psdata = open( fname, O_RDONLY )) != -1 ){ 
-		read(psdata, (void *) &psbuf, sizeof(struct psinfo) );
-		close(psdata);
-		if(strcmp(psbuf.pr_fname,"nscd") != 0) {
-
-			if(pspid != pid) {
-				pinfo.ap_pid = pspid;
-				pinfo.ap_mask.am_success=class;
-				pinfo.ap_mask.am_failure=class;
-				auditon(A_SETPMASK,(caddr_t)&pinfo,sizeof(pinfo));
+		snprintf(fname,sizeof(fname),"/proc/%s/psinfo",dp->d_name);
+	 	if( (psdata = open( fname, O_RDONLY )) != -1 ) { 
+			read(psdata, (void *) &psbuf, sizeof(struct psinfo) );
+			close(psdata);
+			if(strcmp(psbuf.pr_fname,"nscd") != 0) {
+				if(pspid != pid) {
+					pinfo.ap_pid = pspid;
+					pinfo.ap_mask.am_success=class;
+					pinfo.ap_mask.am_failure=class;
+					auditon(A_SETPMASK,(caddr_t)&pinfo,sizeof(pinfo));
+				} else {
+					// Turn off audit for our own process though!
+					pinfo.ap_pid = pspid;
+					pinfo.ap_mask.am_success=0x00000000;
+					pinfo.ap_mask.am_failure=0x00000000;
+					auditon(A_SETPMASK,(caddr_t)&pinfo,sizeof(pinfo));
+				}
 			}
 		}
-	}
-}*/
-
-
+#else
 		auditon(A_GETPINFO,(caddr_t)&pinfo,sizeof(pinfo));
 
 		// Only turn on auditing for those processes that Solaris has already set an audit mask for.
@@ -767,6 +787,7 @@ int setclass(unsigned int class)
 			pinfo.ap_mask.am_failure=0x00000000;
 			auditon(A_SETPMASK,(caddr_t)&pinfo,sizeof(pinfo));
 		}
+#endif
 	}
 	return(1);
 }
@@ -1804,10 +1825,18 @@ char * eventnumber2name(int number, char * eventname, int buffer)
 ////////////////////////////////////////////////////////////////////////////////
 int sendevent(char *string,int criticality)
 {
+	static int snareseq=1;
 	char stringout[MAX_AUDITREC]="";
 	
 	// Audit record format, including host and SolarisBSM identifier string:
-	snprintf(stringout,MAX_AUDITREC,"%s\tSolarisBSM\t%d\t%s",hostid,criticality,string);
+	snprintf(stringout,MAX_AUDITREC,"%s\tSolarisBSM\t%d\t%s\tsnareseq,%d",hostid,criticality,string,snareseq);
+
+	// increment our snare sequence number
+	snareseq++;
+
+	if(snareseq >= MAXINT) {
+		snareseq=1;
+	}
 	
 	if(AuditDestination & AUDIT_TO_STDOUT) {
 		// Send to STDOUT.
