@@ -14,6 +14,8 @@
 
 #include <strings.h>
 
+#include <errno.h>
+
 #include "webserver.h"
 #include "WebPages.h"
 
@@ -33,6 +35,11 @@ int caught_kill = 0;		/* variable that lets us know when a SIGKILL has been rece
 char SnarePass[256];
 char SnareIP[16];
 
+void webkill(sig) {
+	caught_kill++;
+	return;
+}
+
 int InitWebServer(unsigned short port, char *ip, char *password)
 {
 	struct sockaddr_in local;
@@ -44,6 +51,7 @@ int InitWebServer(unsigned short port, char *ip, char *password)
 	http_message_socket = NULL;
 
 	signal(SIGPIPE,SIG_IGN);
+	// signal(SIGTERM,webkill);
 
 	if (password) {
 		strncpy(SnarePass, password, 256);
@@ -390,7 +398,7 @@ char *GetNextArgument(char *source, char *destvar, int varlength,
 
 	if (*source == '=') {
 		// We have a value. Excellent.
-		*source++;
+		source++;
 
 		while (*source && !(*source == '&' && prevchar != '\\')
 		       && destlen < (vallength-1)) {
@@ -454,7 +462,9 @@ int MatchAuth(char *AuthStart)
 {
 	char AuthString[256]="";
 	char AuthString2[256]="";
+	char TempAuth[256]="";
 	char *pos;
+	char *crypttext;
 	int length=0;
 
 	// Hunt down end of line
@@ -483,6 +493,7 @@ int MatchAuth(char *AuthStart)
 
 	*pos='\0';
 	pos++;
+	strncpy(TempAuth,pos,sizeof(TempAuth));
 
 	if(strcmp(AuthString2,"snare") && strcmp(AuthString2,"Snare") && strcmp(AuthString2,"SNARE")) {
 		// Username does not match
@@ -490,13 +501,33 @@ int MatchAuth(char *AuthStart)
 	}
 
 	// Reuse authstring here.
-	strncpy(AuthString2,(char *)crypt(pos,SALT),sizeof(AuthString2));
+	crypttext=crypt(TempAuth,SALT);
+	if(crypttext) {
+		strncpy(AuthString2,crypttext,sizeof(AuthString2));
+	} else {
+		strncpy(AuthString2,"",sizeof(AuthString2));
+		// Hmm.. fall back to normal crypt.
+		fprintf(stderr,"SNARE Warning: Your version of Solaris does not seem to support BSD MD5 Checksums in crypt.conf\n");
+	}
 
 	if(!strcmp(AuthString2,SnarePass)) {
 		// Password matches
 		return(1);
 	}
 
+	// Do we need a fallback here for plaintext passwords?
+	// If so, how do we guard against a user just entering the md5
+	// checksum as a password?
+
+	// Lets md5 the SnarePass value, and check it against the crypted value.
+	crypttext=crypt(SnarePass,SALT);
+	if(crypttext) {
+		if(!strcmp(AuthString2,crypttext)) {
+			fprintf(stderr,"SNARE Warning: Non-encrypted password used in snare.conf file.\n");
+			return(1);
+		}
+	}
+
 	return(0);
 }
 
@@ -509,23 +540,7 @@ int Send(int sock,char *string,int size) {
 		return(0);
 	}
 
-	/*fd_set wfds;
-	struct timeval tv;
-
-	FD_ZERO(&wfds);
-	FD_SET(sock, &wfds);
-	// Wait up to five microseconds.
-	tv.tv_sec = 0;
-	tv.tv_usec = 5;
-	rc=select(sock+1,NULL,&wfds,NULL,&tv); */
-	//if(rc) {
-		fprintf(stderr,"DEBUG: ABOUT TO SEND...\n");
-		rc=send(sock,string,size,MSG_DONTWAIT);
-		//rc=sendto(sock,string,size,MSG_DONTWAIT,NULL,NULL);
-		fprintf(stderr,"DEBUG: SENT...\n");
-	//} else {
-	//	rc=0;
-	//}
+	rc=send(sock,string,size,MSG_DONTWAIT);
 	return(rc);
 }
 
